#include <SDL/SDL.h>

#include "cMaze.h"
#include "cMazeField.h"
#include "functions.h"
#include "globals.h"
#include "mtwist.h"

int cHuntKill::generateStep() {
    int pM = 0; // pickedMove
    bool stepped = false;
    bool found = false;
    int nX = 0, nY = 0;

    while (!stepped) {
        switch (curState) {
            case HK_HUNT:
                if (curRow >= maze->gSize()) {
                    mazeState = MAZE_PATH;
                    maze->visitD(0, 0);
                    maze->unuse();
                    stepped = true;
                } else {
                    maze->unuse();
                    maze->useRow(curRow);
                    if (maze->rowHasUnvisited(curRow)) { // unvisited in row
                        found = false;
                        for (int x = 0; x < maze->gSize(); x++) {
                            if (maze->hasVisitedN(x, curRow) && maze->isVisited(x, curRow) == false) {
                                maze->pickRandomN(x, curRow, true, &nX, &nY);

                                maze->connect(x, curRow, nX, nY, C_NOWALL);

                                maze->visit(x, curRow);
                                curX = x;
                                curY = curRow;

                                found = true;
                                stepped = true;
                                curState = HK_KILL;
                                break;
                            }
                        }
                        if (!found) {
                            curRow++;
                            minRow++;
                            stepped = true;
                        }
                    } else {  // no unvisited
                        curRow++;
                        minRow++;
                        stepped = true;
                    }
                }
                break;
            case HK_KILL:
                if (maze->hasUnvisitedN(curX, curY)) { // moves available
                    maze->pickRandomN(curX, curY, false, &nX, &nY);

                    maze->connect(curX, curY, nX, nY, C_NOWALL);

                    curX = nX;
                    curY = nY;

                    maze->unuse();
                    maze->use(curX, curY);
                    maze->visit(curX, curY);

                    stepped = true;
                } else {
                    // LET'S HUNT
                    curState = HK_HUNT;
                    curRow = minRow;
                }
                break;
        }
    }

    return 0;
}

cHuntKill::cHuntKill(int size) {
    lastFrame = 0;
    mazeState = MAZE_MGNR;
    pathState = PATH_FIND;
    maze = new cMazeField(size);
    surfEnlarged = SDL_CreateRGBSurface(SDL_SWSURFACE, 500, 500, screen->format->BitsPerPixel, screen->format->Rmask, screen->format->Gmask, screen->format->Bmask, screen->format->Amask);

    curX = 0;
    curY = 0;
    maze->visit(curX, curY);
    curState = HK_KILL;
    minRow = 0;

    dAddFront(0, 0);
}

cHuntKill::~cHuntKill() {
    delete maze;

    for (int x = 0; x < dFronts.size(); x++) {
        dFronts[x].clear();
    }
    dFronts.clear();

    SDL_FreeSurface(surfEnlarged);
}

/*
                                if (neighbours.size() > 0) {
                                    for (int i = 0; i < neighbours.size(); i++) {
                                        neighbours[i].clear();
                                    }
                                    neighbours.clear();
                                }

                                // above. pFX = 0 || < size
                                if (inRange(x, 0, maze->gSize()) && inRange(curRow - 1, 0, maze->gSize())) {
                                    if (maze->isVisited(x, curRow - 1)  == true) {
                                        tVector.clear();
                                        tVector.push_back(x);
                                        tVector.push_back(curRow - 1);
                                        neighbours.push_back(tVector);
                                    }
                                }

                                // under
                                if (inRange(x, 0, maze->gSize()) && inRange(curRow + 1, 0, maze->gSize())) {
                                    if (maze->isVisited(x, curRow + 1)  == true) {
                                        tVector.clear();
                                        tVector.push_back(x);
                                        tVector.push_back(curRow + 1);
                                        neighbours.push_back(tVector);
                                    }
                                }

                                // left
                                if (inRange(x - 1, 0, maze->gSize()) && inRange(curRow, 0, maze->gSize())) {
                                    if (maze->isVisited(x - 1, curRow)  == true) {
                                        tVector.clear();
                                        tVector.push_back(x - 1);
                                        tVector.push_back(curRow);
                                        neighbours.push_back(tVector);
                                    }
                                }

                                //right
                                if (inRange(x, 0, maze->gSize()) && inRange(curRow, 0, maze->gSize())) {
                                    if (maze->isVisited(x, curRow) == true) {
                                        tVector.clear();
                                        tVector.push_back(x + 1);
                                        tVector.push_back(curRow);
                                        neighbours.push_back(tVector);
                                    }
                                }

                                pM = mt_lrand() % neighbours.size();
                                */

                    /*
                    if (neighbours.size() > 0) {
                        for (int i = 0; i < neighbours.size(); i++) {
                            neighbours[i].clear();
                        }
                        neighbours.clear();
                    }

                    // above. pFX = 0 || < size
                    if (inRange(curX, 0, maze->gSize()) && inRange(curY - 1, 0, maze->gSize())) {
                        if (maze->isVisited(curX, curY - 1)  == false) {
                            tVector.clear();
                            tVector.push_back(curX);
                            tVector.push_back(curY - 1);
                            neighbours.push_back(tVector);
                        }
                    }

                    // under
                    if (inRange(curX, 0, maze->gSize()) && inRange(curY + 1, 0, maze->gSize())) {
                        if (maze->isVisited(curX, curY + 1)  == false) {
                            tVector.clear();
                            tVector.push_back(curX);
                            tVector.push_back(curY + 1);
                            neighbours.push_back(tVector);
                        }
                    }

                    // left
                    if (inRange(curX - 1, 0, maze->gSize()) && inRange(curY, 0, maze->gSize())) {
                        if (maze->isVisited(curX - 1, curY)  == false) {
                            tVector.clear();
                            tVector.push_back(curX - 1);
                            tVector.push_back(curY);
                            neighbours.push_back(tVector);
                        }
                    }

                    //right
                    if (inRange(curX + 1, 0, maze->gSize()) && inRange(curY, 0, maze->gSize())) {
                        if (maze->isVisited(curX + 1, curY) == false) {
                            tVector.clear();
                            tVector.push_back(curX + 1);
                            tVector.push_back(curY);
                            neighbours.push_back(tVector);
                        }
                    }

                    pM = mt_lrand() % neighbours.size();
                    */
